package models

import (
	"eta/eta_index_lib/utils"
	"fmt"
	"strconv"
	"strings"
	"time"

	"github.com/beego/beego/v2/client/orm"
)

type BaseFromSmmData struct {
	SmmDataId          int `orm:"column(smm_data_id);pk"`
	BaseFromSmmIndexId int
	IndexCode          string
	DataTime           string
	Value              string
	CreateTime         time.Time
	ModifyTime         time.Time
	DataTimestamp      int64
}

func AddBaseFromSmmData(item []*BaseFromSmmData) (err error) {
	o := orm.NewOrm()
	_, err = o.InsertMulti(500, item)
	return
}

func GetBaseFromSmmDataByCondition(condition string, pars []interface{}) (list []*BaseFromSmmData, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_smm_data WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	_, err = o.Raw(sql, pars).QueryRows(&list)
	return
}

// 新增有色指标数据
func AddEdbDataFromSmm(edbCode string, smmBaseDataAll []*BaseFromSmmData) (err error) {
	o := orm.NewOrm()
	var isAdd bool
	addSql := ` INSERT INTO edb_data_ys(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	existMap := make(map[string]string)
	for _, sv := range smmBaseDataAll {
		eDate := sv.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, ok := existMap[eDate]; !ok {
			addSql += GetAddSql("0", edbCode, eDate, timeStr, sv.Value)
			isAdd = true
		}
		existMap[eDate] = sv.Value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

// RefreshEdbDataFromSmm 刷新有色指标数据
func RefreshEdbDataFromSmm(edbInfoId int, edbCode, startDate string) (err error) {
	source := utils.DATA_SOURCE_YS
	subSource := utils.DATA_SUB_SOURCE_EDB

	o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	smmDataList, err := GetBaseFromSmmDataByCondition(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, subSource, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_ys(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range smmDataList {
		item := v
		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			sValue := item.Value

			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, subSource, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, subSource, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			fmt.Println("RefreshEdbDataFromSmm add Err", err.Error())
			return
		}
	}
	return
}

type BaseFromSmmIndex struct {
	BaseFromSmmIndexId int64 `orm:"column(base_from_smm_index_id);pk"`
	ClassifyId         int
	Interface          string
	Name               string
	IndexCode          string
	IndexName          string
	Type1              string `orm:"column(type_1)"`
	Type2              string `orm:"column(type_2)"`
	Type3              string `orm:"column(type_3)"`
	Frequency          string
	Unit               string
	ApiStartTime       string
	ApiUpdateTime      string
	StartTime          string
	FinishTime         string
	BaseFileName       string
	RenameFileName     string
	StartDate          string
	EndDate            string
	TerminalCode       string `description:"编码"`
	CreateTime         time.Time
	ModifyTime         time.Time
	DataState          string
	ReleaseTime        string
}

func (d *BaseFromSmmIndex) AddBaseFromSmmIndex() (lastId int64, err error) {
	o := orm.NewOrm()
	lastId, err = o.Insert(d)
	return
}

func (d *BaseFromSmmIndex) GetSmmIndexItem(indexCode string) (item *BaseFromSmmIndex, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_smm_index WHERE index_code=? `

	fmt.Println(sql, indexCode)
	err = o.Raw(sql, indexCode).QueryRow(&item)
	return
}

// Add 新增
func (m *BaseFromSmmIndex) Add() (err error) {
	o := orm.NewOrm()
	_, err = o.Insert(m)
	return
}

func GetBaseFromSmmDataBySmmCode(smmCode string) (list []*BaseFromSmmData, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_smm_data WHERE index_code=? `
	_, err = o.Raw(sql, smmCode).QueryRows(&list)
	return
}

func ModifySmmIndexSort(baseFromSmmIndexId int64) (err error) {
	o := orm.NewOrm()
	sql := ` UPDATE base_from_smm_index SET sort=? WHERE base_from_smm_index_id=? `
	_, err = o.Raw(sql, baseFromSmmIndexId, baseFromSmmIndexId).Exec()
	return
}

func GetSmmIndexInfoMaxAndMinInfo(indexCode string) (item *EdbInfoMaxAndMinInfo, err error) {
	o := orm.NewOrm()
	sql := ` SELECT MIN(data_time) AS min_date,MAX(data_time) AS max_date,MIN(value) AS min_value,MAX(value) AS max_value FROM base_from_smm_data WHERE index_code=? `
	err = o.Raw(sql, indexCode).QueryRow(&item)
	if err != nil {
		return
	}

	// 获取最新值
	var latest_value float64
	sql = ` SELECT value AS latest_value FROM base_from_smm_data WHERE index_code=? ORDER BY data_time DESC LIMIT 1 `
	err = o.Raw(sql, indexCode).QueryRow(&latest_value)
	if err != nil {
		return
	}
	item.LatestValue = latest_value

	return
}

func ModifySmmIndexMaxAndMinInfo(indexCode string, item *EdbInfoMaxAndMinInfo) (err error) {
	o := orm.NewOrm()
	sql := ` UPDATE base_from_smm_index SET start_date=?,end_date=?,end_value=?,modify_time=NOW() WHERE index_code=? `
	_, err = o.Raw(sql, item.MinDate, item.MaxDate, item.LatestValue, indexCode).Exec()
	return
}

// HandleSmmExcelDataReq SMM的excel数据
type HandleSmmExcelDataReq struct {
	BaseFilePath   string
	RenameFilePath string
	IndexName      string
	Source         string
	IndexCode      string
	Unit           string
	Frequency      string
	ExcelDataMap   map[string]string
	TerminalCode   string `description:"编码"`
}

// MultiUpdateBaseFromSmmDataValue 批量更新SMM指标数据
func MultiUpdateBaseFromSmmDataValue(items []*BaseFromSmmData) (err error) {
	if len(items) == 0 {
		return
	}

	o := orm.NewOrm()
	sql := `UPDATE base_from_smm_data SET value = ?, modify_time = NOW() WHERE index_code = ? AND data_time = ? LIMIT 1`
	p, err := o.Raw(sql).Prepare()
	if err != nil {
		return
	}
	defer func() {
		_ = p.Close()
	}()
	for _, v := range items {
		if v.IndexCode == "" || v.DataTime == "" {
			continue
		}
		_, err = p.Exec(v.Value, v.IndexCode, v.DataTime)
		if err != nil {
			return
		}
	}
	return
}

type AddSmmIndexToDataSourceResp struct {
	EdbCode         string `description:"指标编码"`
	ClassifyId      int    `description:"指标分类"`
	SysUserId       int    `description:"操作人id"`
	SysUserRealName string `description:"操作人真实名称"`
}

func (m *BaseFromSmmIndex) GetIndexItem(indexCode string) (item *BaseFromSmmIndex, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_smm_index WHERE index_code = ? `
	err = o.Raw(sql, indexCode).QueryRow(&item)
	return
}

// 修改
func (m *BaseFromSmmIndex) Update(updateParams, whereParam map[string]interface{}) (err error) {
	to := orm.NewOrm()
	ptrStructOrTableName := "base_from_smm_index"

	qs := to.QueryTable(ptrStructOrTableName)
	for expr, exprV := range whereParam {
		qs = qs.Filter(expr, exprV)
	}
	_, err = qs.Update(updateParams)
	return
}

// BridgeZhongJiIndexDataParams 桥接服务-获取中基指标数据入参
type BridgeZhongJiIndexDataParams struct {
	EdbCode string `json:"edb_code" form:"edb_code" description:"指标编码"`
}

type BridgeZhongJiIndexListParams struct {
	Page     int `json:"page" form:"page" description:"页码"`
	PageSize int `json:"page_size" form:"page_size" description:"每页指标个数，每页不能超过500条，超过会固定为500"`
}

type BaseFromSmmDataList struct {
	Data      []*BaseFromSmmData
	IndexCode string
}

type UpdateRule struct {
	RuleDesc    string         `json:"rule_desc"`
	Notice      string         `json:"notice"`
	ReleaseTime string         `json:"release_time"`
	RuleInfo    UpdateRuleInfo `json:"rule_info"`
}

type UpdateRuleInfo struct {
	UpdCal       string `json:"upd_cal"`
	UpdCalUnit   string `json:"upd_cal_unit"`
	PeriodOffset int    `json:"period_offset"`
	UpdCount     int    `json:"upd_count"`
}

type DataItem struct {
	QuotaID     string     `json:"quota_id"`
	QuotaName   string     `json:"quota_name"`
	Mark        string     `json:"mark"`
	Unit        string     `json:"unit"`
	ValueType   string     `json:"value_type"`
	Source      string     `json:"source"`
	ListOrder   int        `json:"list_order"`
	DataStart   string     `json:"data_start"`
	DataEnd     string     `json:"data_end"`
	Frequency   string     `json:"frequency"`
	Precision   int        `json:"precision"`
	TypeCode    string     `json:"type_code"`
	TypeName    string     `json:"type_name"`
	TypeCodeAll string     `json:"type_code_all"`
	TypeAll     string     `json:"type_all"`
	Tags        string     `json:"tags"`
	AuthStart   string     `json:"auth_start"`
	AuthFinish  string     `json:"auth_finish"`
	AuthLang    string     `json:"auth_lang"`
	AuthModule  string     `json:"auth_module"`
	DataDur     string     `json:"data_dur"`
	Spec        string     `json:"spec"`
	Area        string     `json:"area"`
	DataState   string     `json:"data_state"`
	UpdateRule  UpdateRule `json:"update_rule"`
}

type ListData struct {
	Total       int        `json:"total"`
	CurrentPage int        `json:"current_page"`
	PageSize    int        `json:"page_size"`
	PageCount   int        `json:"page_count"`
	Data        []DataItem `json:"data"`
}

type SmmIndexListResponse struct {
	Code int      `json:"code"`
	Msg  string   `json:"msg"`
	Data ListData `json:"data"`
}

type SmmListResp struct {
	Code int                  `json:"code"`
	Msg  string               `json:"msg"`
	Data SmmIndexListResponse `json:"data"`
}

func AddBaseFromSmmIndex(list []*BaseFromSmmIndex) (lastId int64, err error) {
	o := orm.NewOrm()
	_, err = o.InsertMulti(len(list), list)
	return
}

// BridgeZhongJiIndexLatestDataParams 桥接服务-获取中基更新指标数据入参
type BridgeZhongJiIndexLatestDataParams struct {
	EdbCode  string `json:"edb_code" form:"edb_code" description:"指标编码"`
	LastTime int64  `json:"last_time" form:"last_time" description:"上次更新时间戳（秒）"`
}

func GetBaseFromSmmIndex() (list []*BaseFromSmmIndex, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_smm_index`
	_, err = o.Raw(sql).QueryRows(&list)
	return
}

func ModifyBaseFromSmmIndex(item *BaseFromSmmIndex) (err error) {
	o := orm.NewOrm()
	sql := ` UPDATE base_from_smm_index SET end_date = ?, modify_time=NOW(), data_state= ? `
	_, err = o.Raw(sql, item.EndDate, item.DataState).Exec()
	return
}

func (m *BaseFromSmmIndex) UpdateCols(cols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(m, cols...)
	return
}

func ModifyBaseFromSmmIndexDate(item *BaseFromSmmIndex) (err error) {
	o := orm.NewOrm()
	sql := ` UPDATE base_from_smm_index SET end_date = ?, modify_time=NOW(), data_state= ? WHERE base_from_smm_index_id=?`
	_, err = o.Raw(sql, item.EndDate, item.DataState, item.BaseFromSmmIndexId).Exec()
	return
}

type SmmLatestDataResponse struct {
	Code int           `json:"code"`
	Msg  string        `json:"msg"`
	Data SmmLatestResp `json:"data"`
}

type SmmLatestResp struct {
	Code int           `json:"code"`
	Msg  string        `json:"msg"`
	Data SmmLatestData `json:"data"`
}

type SmmLatestData struct {
	DataLen  int     `json:"data_len"`
	DataList []Datum `json:"data_list"`
}

type Datum struct {
	QuotaID string `json:"quota_id"`
	Date    string `json:"date"`
	Value   string `json:"value"`
	Mark    string `json:"mark"`
	Create  string `json:"create"`
	Update  string `json:"update"`
}

// RefreshEdbDataFromSmmToEdb 刷新有色指标数据
func RefreshEdbDataFromSmmToEdb(edbInfoId int, edbCode, startDate string, smmDataList []*BaseFromSmmData) (err error) {
	source := utils.DATA_SOURCE_YS
	subSource := utils.DATA_SUB_SOURCE_EDB

	o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, subSource, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_ys(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range smmDataList {
		item := v
		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			sValue := item.Value

			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, subSource, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, subSource, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			fmt.Println("RefreshEdbDataFromSmm add Err", err.Error())
			return
		}
	}
	return
}
